DECLARE SUB equal.long.func (strVariable AS STRING, intVariable AS INTEGER)
DEFINT A-Z

CONST cstTrue = -1, cstFALSE = 0

'$INCLUDE: 'asm.bi'
'$INCLUDE: 'const.bi'
'$INCLUDE: 'expr.bi'
'$INCLUDE: 'is.bi'
'$INCLUDE: 'keyword.bi'
'$INCLUDE: 'label.bi'
'$INCLUDE: 'token.bi'

DIM SHARED strToken           AS STRING
DIM SHARED intToken           AS INTEGER
DIM SHARED intSharedExprIndex AS INTEGER

SUB equal.byte (strVariable AS STRING)

'#############################################################################
'# Byte Variable = Immediate [ byte | integer ]                              #
'#############################################################################

expr.token
IF strToken = "" THEN
    expr.putback

    SELECT CASE intToken
  
        CASE cstImmediateByte, cstImmediateInteger
             SELECT CASE VAL(strToken)
                
                 CASE IS < -128
                    expr.error "Error, Underflow."
                    END
                
                 CASE IS > 127
                    expr.error "Error, Overflow."
                    END

             END SELECT

             asm.code "mov " + strVariable + ", " + strToken
             EXIT SUB

        CASE cstImmediateLong
             expr.error "Error, Overflow."
             END

        CASE cstImmediateDouble, cstImmediateSingle
             expr.error "Error, Overflow."
             END

        CASE cstImmediateString
             expr.error "Error, type-mismatch"
             END

    END SELECT


'#############################################################################
'# Byte Variable = Variable [ byte | integer ]                               #
'#############################################################################

    SELECT CASE intToken

        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "mov " + strVariable + ", al"

        CASE cstVariableInteger
             expr.error "Error, type-mismatch"
             END
      
        CASE cstVariableDouble, cstVariableSingle
             expr.error "Error, type-mismatch"
             END

        CASE cstVariableLong, cstVariableString
             expr.error "Error, type-mismatch"
             END

    END SELECT
END IF

'#############################################################################
'# Byte Variable = term [op] term                                            #
'#############################################################################

expr.putback

DIM strOp        AS STRING
DIM strVariable2 AS STRING

    strVariable2 = strToken

    expr.token
    strOp = strToken
  
    expr.token

    signed.byte.filter

  
    SELECT CASE strOp
      
        CASE "^"
             signed.byte.exp strVariable, strVariable2

        CASE "*"
             signed.byte.mul strVariable, strVariable2

        CASE "/"
             signed.byte.fdiv strVariable, strVariable2

        CASE "\"
             signed.byte.idiv strVariable, strVariable2
         
        CASE "mod"
             signed.byte.mod strVariable, strVariable2

        CASE "+"
             signed.byte.add strVariable, strVariable2
                                                      
        CASE "-"
             signed.byte.sub strVariable, strVariable2

    END SELECT


END SUB

SUB equal.int (strVariable AS STRING)

'#############################################################################
'# Integer Variable = Immediate [ byte | integer ]                           #
'#############################################################################

expr.token
IF strToken = "" THEN
    expr.putback

    SELECT CASE intToken
   
        CASE cstImmediateByte, cstImmediateInteger
             asm.code "mov " + strVariable + ", " + strToken
             EXIT SUB

        CASE cstImmediateLong
             expr.error "Error, Overflow."
             END

    END SELECT

'#############################################################################
'# Integer Variable = Variable [ byte | integer ]                            #
'#############################################################################
    SELECT CASE intToken

        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "mov " + strVariable + ", ax"

        CASE cstVariableInteger
             asm.code "mov ax, " + strToken
             asm.code "mov " + strVariable + ", ax"
       
        CASE cstVariableDouble, cstVariableSingle
             expr.error "Error, type-mismatch"
             END

        CASE cstVariableLong, cstVariableString
             expr.error "Error, type-mismatch"
             END

    END SELECT
END IF

'#############################################################################
'# Integer Variable = term [op] term                                         #
'#############################################################################

expr.putback

DIM strOp        AS STRING
DIM strVariable2 AS STRING

    strVariable2 = strToken

    expr.token
    strOp = strToken
   
    expr.token

    signed.int.filter

   
    SELECT CASE strOp
       
        CASE "^"
             signed.int.exp strVariable, strVariable2

        CASE "*"
             signed.int.mul strVariable, strVariable2

        CASE "/"
             signed.int.fdiv strVariable, strVariable2

        CASE "\"
             signed.int.idiv strVariable, strVariable2
          
        CASE "mod"
             signed.int.mod strVariable, strVariable2

        CASE "+"
             signed.int.add strVariable, strVariable2
                                                       
        CASE "-"
             signed.int.sub strVariable, strVariable2

    END SELECT

END SUB

SUB equal.long (strVariable AS STRING)

'#############################################################################
'# Long Variable = Function                                                  #
'#############################################################################

    SELECT CASE intToken

        CASE cstProcedureFunction
             equal.long.func strVariable, cstVariableLong
      
        CASE cstProcedureSub
            expr.error "Error, Sub can not be used in an expression."
            END

    END SELECT


'#############################################################################
'# Long Variable = Immediate [ byte | integer | Long ]                       #
'#############################################################################
expr.token
IF strToken = "" THEN
    expr.putback

    SELECT CASE intToken
  
        CASE cstImmediateByte, cstImmediateInteger
             asm.code "mov " + strVariable + ", " + strToken
             EXIT SUB

        CASE cstImmediateLong
             asm.code "mov " + strVariable + ", " + strToken
             EXIT SUB
       
        CASE cstImmediateDouble, cstImmediateSingle
             expr.error "Error, type-mismatch"
             END

        CASE cstImmediateString
             expr.error "Error, type-mismatch"
             END

    END SELECT

'#############################################################################
'# Long Variable = Variable [ byte | integer | Long ]                        #
'#############################################################################

    SELECT CASE intToken

        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "cwd"
             asm.code "mov " + strVariable + ", eax"

        CASE cstVariableInteger
             asm.code "mov ax, " + strToken
             asm.code "cwd"
             asm.code "mov " + strVariable + ", eax"
      
        CASE cstVariableLong
             asm.code "mov eax, " + strToken
             asm.code "mov " + strVariable + ", eax"
       
        CASE cstVariableDouble, cstVariableSingle
             expr.error "Error, type-mismatch"
             END

        CASE cstVariableString
             expr.error "Error, type-mismatch"
             END

    END SELECT
END IF

'#############################################################################
'# long Variable = term [op] term                                            #
'#############################################################################

expr.putback

DIM strOp        AS STRING
DIM strVariable2 AS STRING

    strVariable2 = strToken

    expr.token
    strOp = strToken
  
    expr.token

    signed.lng.filter

  
    SELECT CASE strOp
      
        CASE "^"
             signed.lng.exp strVariable, strVariable2

        CASE "*"
             signed.lng.mul strVariable, strVariable2

        CASE "/"
             signed.lng.fdiv strVariable, strVariable2

        CASE "\"
             signed.lng.idiv strVariable, strVariable2
         
        CASE "mod"
             signed.lng.mod strVariable, strVariable2

        CASE "+"
             signed.lng.add strVariable, strVariable2
                                                      
        CASE "-"
             signed.lng.sub strVariable, strVariable2

    END SELECT


END SUB

'#############################################################################
'# Please note that this routine is poorly designed and as a result doesn't  #
'# operate in a manner suitable to the standard BASIC we all know and love.  #
'#############################################################################
SUB equal.long.func (strVariable AS STRING, intVariable AS INTEGER)

DIM strFunction  AS STRING
DIM strTemporary AS STRING

strFunction = strToken

expr.token

    IF strToken = "(" THEN
      
        strTemporary = label.param$(strFunction)
        intIdx = VAL(MID$(strTemporary, 1, INSTR(1, strTemporary, "|") - 1))
   
        DO
           
            expr.token

            IF strToken = ")" THEN EXIT DO
          
            IF strToken = "," THEN
               intIdx = intIdx + 1
               intX = intX - 10
            ELSE
                SELECT CASE intToken

                    CASE cstVariableByte, cstVariableInteger
                    CASE cstVariableLong

                    CASE cstImmediateByte, cstImmediateInteger
                         asm.code "push " + strToken
                         

                    CASE cstImmediateLong
                   
                END SELECT
            END IF
            intX = intX + 5
           
        LOOP
        asm.code "call " + strFunction
        asm.code "mov " + LCASE$(strVariable) + ", eax"
            
        
    ELSE
        asm.code "call " + strFunction
        asm.code "mov " + LCASE$(strVariable) + ", eax"
    END IF


END SUB

SUB expr.equal (strVariable AS STRING, intVariable AS INTEGER)

expr.token

    SELECT CASE intVariable

        CASE cstVariableByte
             equal.byte strVariable

        CASE cstVariableDouble, cstVariableSingle
             expr.error "Double/Single not completely implimented in this version."
             END
       
        CASE cstVariableInteger
             equal.int strVariable

        CASE cstVariableLong
             equal.long strVariable

    END SELECT

END SUB

'#############################################################################
'#                                                                           #
'#############################################################################
SUB expr.error (strSource AS STRING)
   
    PRINT strSource
    PRINT tkn.line.index

END SUB

SUB expr.parse (intTknIdx AS INTEGER)

    intSharedExprIndex = intTknIdx

DIM strTemp   AS STRING
DIM strChar   AS STRING
DIM intIndex  AS INTEGER
   
    expr.token
        IF strToken = "" THEN
           expr.error "Expected expression."
           END
        END IF

'#############################################################################
'# Currently only allows simple math.                                        #
'#############################################################################
'# [variable] = [variable | immediate | function]                            #
'# x = y | x = 4 | x = int(y)                                                #
'# [variable] = [variable] [*|/|\|mod|+|-] [variable | immediate | function] #
'# x = x + y | x = x - 1 | etc                                               #
'#############################################################################
    
    SELECT CASE intToken

        CASE cstVariableByte
        CASE cstVariableDouble
        CASE cstVariableInteger
        CASE cstVariableLong
        CASE cstVariableSingle
        CASE cstVariableString
        'Add support for unsinged [byte | integer | long] variables here.

        CASE ELSE
            expr.error "Expected variable"
            END

    END SELECT

DIM strVariable AS STRING
DIM intVariable AS INTEGER

    strVariable = strToken
    intVariable = intToken

    expr.token

    SELECT CASE strToken

        CASE "="
             expr.equal strVariable, intVariable

        CASE "=="
             'expr.compare strVariable, intVariable
   
    END SELECT


END SUB

SUB expr.putback
   
    intSharedExprIndex = intSharedExprIndex - 2
    expr.token

END SUB

SUB expr.token
   
    strToken = ""
    intToken = 0

    strToken = tkn.item(intSharedExprIndex)
    intSharedExprIndex = intSharedExprIndex + 1
   
        SELECT CASE is.procedure(strToken)
          
            CASE cstProcedureFunction
                 intToken = cstProcedureFunction
                 EXIT SUB
          
            CASE cstProcedureSub
                 intToken = cstProcedureSub
                 EXIT SUB

        END SELECT

        SELECT CASE is.variable(strToken)

            CASE cstVariableByte
                 intToken = cstVariableByte
                 EXIT SUB
           
            CASE cstVariableDouble
                 intToken = cstVariableDouble
                 EXIT SUB
           
            CASE cstVariableInteger
                 intToken = cstVariableInteger
                 EXIT SUB
           
            CASE cstVariableLong
                 intToken = cstVariableLong
                 EXIT SUB
           
            CASE cstVariableSingle
                 intToken = cstVariableSingle
                 EXIT SUB
           
            CASE cstVariableString
                 intToken = cstVariableString
                 EXIT SUB
           
            'Add support for unsinged [byte | integer | long] variables here.

        END SELECT

        SELECT CASE is.delim(strToken)

            CASE cstTrue
                 intToken = cstDelimiter
                 EXIT SUB
            
        END SELECT

        SELECT CASE is.immediate(strToken)

            CASE cstImmediateByte
                 intToken = cstImmediateByte
                 EXIT SUB
           
            CASE cstImmediateDouble
                 intToken = cstImmediateDouble
                 EXIT SUB
           
            CASE cstImmediateInteger
                 intToken = cstImmediateInteger
                 EXIT SUB
           
            CASE cstImmediateLong
                 intToken = cstImmediateLong
                 EXIT SUB
           
            CASE cstImmediateSingle
                 intToken = cstImmediateSingle
                 EXIT SUB
           
            CASE cstImmediateString
                 intToken = cstImmediateString
                 EXIT SUB
            'Add support for unsinged [byte | integer | long] immediates here.

        END SELECT

END SUB

SUB signed.byte.add (strVariable AS STRING, strVariable2 AS STRING)
   
    SELECT CASE intToken
            
        CASE cstImmediateByte, cstImmediateInteger
            
             SELECT CASE VAL(strToken)
               
                 CASE IS < -128
                    expr.error "Error, Underflow."
                    END
               
                 CASE IS > 127
                    expr.error "Error, Overflow."
                    END

             END SELECT

                 asm.code "mov al, " + strVariable2
                 asm.code "add al, " + strToken
                 asm.code "mov " + strVariable + ", al"
                 asm.code "jo lbl_Signed_Overflow"


        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "add al, " + strVariable2
             asm.code "mov " + strVariable + ", al"
             asm.code "jo lbl_Signed_Overflow"
                      
    END SELECT

END SUB

SUB signed.byte.exp (strVariable AS STRING, strVariable2 AS STRING)
END SUB

SUB signed.byte.fdiv (strVariable AS STRING, strVariable2 AS STRING)
END SUB

'#############################################################################
'# Filter out all possible compile time errors.              Byte routine    #
'#############################################################################
SUB signed.byte.filter

END SUB

SUB signed.byte.idiv (strVariable AS STRING, strVariable2 AS STRING)
   
    SELECT CASE intToken
           
        CASE cstImmediateByte, cstImmediateInteger
           
             SELECT CASE VAL(strToken)
              
                 CASE IS < -128
                    expr.error "Error, Underflow."
                    END
              
                 CASE IS > 127
                    expr.error "Error, Overflow."
                    END

             END SELECT
                
                 asm.code "mov al, " + strToken
                 asm.code "mov bl, " + strVariable2
                 asm.code "idiv bl"
                 asm.code "mov " + strVariable + ", al"
                 asm.code "jo lbl_Signed_Overflow"
       
        CASE cstVariableByte
                 asm.code "mov al, " + strToken
                 asm.code "mov bl, " + strVariable2
                 asm.code "idiv bl"
                 asm.code "mov " + strVariable + ", al"
                 asm.code "jo lbl_Signed_Overflow"

    END SELECT

END SUB

SUB signed.byte.mod (strVariable AS STRING, strVariable2 AS STRING)
   
    SELECT CASE intToken
          
        CASE cstImmediateByte, cstImmediateInteger
          
             SELECT CASE VAL(strToken)
             
                 CASE IS < -128
                    expr.error "Error, Underflow."
                    END
             
                 CASE IS > 127
                    expr.error "Error, Overflow."
                    END

             END SELECT
               
                 asm.code "mov al, " + strToken
                 asm.code "mov bl, " + strVariable2
                 asm.code "idiv bl"
                 asm.code "mov " + strVariable + ", dl"
                 asm.code "jo lbl_Signed_Overflow"
      
        CASE cstVariableByte
                 asm.code "mov al, " + strToken
                 asm.code "mov bl, " + strVariable2
                 asm.code "idiv bl"
                 asm.code "mov " + strVariable + ", dl"
                 asm.code "jo lbl_Signed_Overflow"

    END SELECT

END SUB

SUB signed.byte.mul (strVariable AS STRING, strVariable2 AS STRING)
   
    SELECT CASE intToken
           
        CASE cstImmediateByte, cstImmediateInteger
            
             SELECT CASE VAL(strToken)
              
                 CASE IS < -128
                    expr.error "Error, Underflow."
                    END
              
                 CASE IS > 127
                    expr.error "Error, Overflow."
                    END

             END SELECT

             asm.code "mov al, " + strVariable2
             asm.code "imul " + strToken
             asm.code "mov " + strVariable + ", al"
             asm.code "jo lbl_Signed_Overflow"

        CASE cstVariableByte
             asm.code "mov al, " + strVariable2
             asm.code "imul " + strToken
             asm.code "mov " + strVariable + ", al"
             asm.code "jo lbl_Signed_Overflow"
                 
                    
    END SELECT

END SUB

SUB signed.byte.sub (strVariable AS STRING, strVariable2 AS STRING)
   
    SELECT CASE intToken
           
        CASE cstImmediateByte, cstImmediateInteger
           
             SELECT CASE VAL(strToken)
              
                 CASE IS < -128
                    expr.error "Error, Underflow."
                    END
              
                 CASE IS > 127
                    expr.error "Error, Overflow."
                    END

             END SELECT

                 asm.code "mov al, " + strVariable2
                 asm.code "sub al, " + strToken
                 asm.code "mov " + strVariable + ", al"
                 asm.code "jo lbl_Signed_Overflow"

        CASE cstVariableByte
                 asm.code "mov al, " + strVariable2
                 asm.code "sub al, " + strToken
                 asm.code "mov " + strVariable + ", al"
                 asm.code "jo lbl_Signed_Overflow"
   
    END SELECT

END SUB

SUB signed.int.add (strVariable AS STRING, strVariable2 AS STRING)
          
    SELECT CASE intToken
             
        CASE cstImmediateByte, cstImmediateInteger, cstVariableInteger
             asm.code "mov ax, " + strVariable2
             asm.code "add ax, " + strToken
             asm.code "mov " + strVariable + ", ax"
             asm.code "jo lbl_Signed_Overflow"

        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "add ax, " + strVariable2
             asm.code "mov " + strVariable + ", ax"
             asm.code "jo lbl_Signed_Overflow"
                       
    END SELECT

END SUB

SUB signed.int.exp (strVariable AS STRING, strVariable2 AS STRING)

    expr.error "^ not implimented in this version."
    END

END SUB

SUB signed.int.fdiv (strVariable AS STRING, strVariable2 AS STRING)
            
    expr.error "FPU division not implimented in this version."
    END

END SUB

'#############################################################################
'# Filter out all possible compile time errors.              Integer routine #
'#############################################################################
SUB signed.int.filter
   
    SELECT CASE intToken

        CASE cstImmediateLong
             expr.error "Error, Overflow."
             END
                    
        CASE cstImmediateDouble, cstImmediateSingle
             expr.error "Error, type-mismatch"
             END

        CASE cstImmediateString
             expr.error "Error, type-mismatch"
             END
      
        CASE cstVariableLong
             expr.error "Error, Overflow."
             END
                   
        CASE cstVariableDouble, cstVariableSingle
             expr.error "Error, type-mismatch"
             END

        CASE cstVariableString
             expr.error "Error, type-mismatch"
             END
 
    END SELECT

END SUB

SUB signed.int.idiv (strVariable AS STRING, strVariable2 AS STRING)

    SELECT CASE intToken
           
        CASE cstImmediateByte, cstImmediateInteger, cstVariableInteger
             asm.code "mov ax, " + strVariable2
             asm.code "cwd"
             asm.code "idiv " + strToken
             asm.code "mov " + strVariable + ", ax"
             asm.code "jo lbl_Signed_Overflow"
                  
        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "mov bx, ax"
             asm.code "mov ax, " + strVariable2
             asm.code "cwd"
             asm.code "idiv bx"
             asm.code "mov " + strVariable + ", ax"
             asm.code "jo lbl_Signed_Overflow"
                    
    END SELECT

END SUB

SUB signed.int.mod (strVariable AS STRING, strVariable2 AS STRING)
           
    SELECT CASE intToken
          
        CASE cstImmediateByte, cstImmediateInteger, cstVariableInteger
             asm.code "mov ax, " + strVariable2
             asm.code "cwd"
             asm.code "idiv " + strToken
             asm.code "mov " + strVariable + ", dx"
             asm.code "jo lbl_Signed_Overflow"

        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "mov bx, ax"
             asm.code "mov ax, " + strVariable2
             asm.code "cwd"
             asm.code "idiv bx"
             asm.code "mov " + strVariable + ", dx"
             asm.code "jo lbl_Signed_Overflow"

    END SELECT

END SUB

SUB signed.int.mul (strVariable AS STRING, strVariable2 AS STRING)
           
    SELECT CASE intToken
            
        CASE cstImmediateByte, cstImmediateInteger, cstVariableInteger
             asm.code "mov ax, " + strVariable2
             asm.code "imul " + strToken
             asm.code "mov " + strVariable + ", ax"
             asm.code "jo lbl_Signed_Overflow"

        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "mov bx, ax"
             asm.code "mov ax, " + strVariable2
             asm.code "imul bx"
             asm.code "mov " + strVariable + ", ax"
             asm.code "jo lbl_Signed_Overflow"
                  
                     
    END SELECT

END SUB

SUB signed.int.sub (strVariable AS STRING, strVariable2 AS STRING)
   
    SELECT CASE intToken
            
        CASE cstImmediateByte, cstImmediateInteger, cstVariableInteger
             asm.code "mov ax, " + strVariable2
             asm.code "sub ax, " + strToken
             asm.code "mov " + strVariable + ", ax"
             asm.code "jo lbl_Signed_Overflow"
                  
        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "mov bx, ax"
             asm.code "mov ax, " + strVariable2
             asm.code "sub ax, bx"
             asm.code "mov " + strVariable + ", ax"
             asm.code "jo lbl_Signed_Overflow"

    END SELECT

END SUB

SUB signed.lng.add (strVariable AS STRING, strVariable2 AS STRING)
         
    SELECT CASE intToken
            
        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "cwd"
             asm.code "add eax, " + strVariable2
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"

        CASE cstVariableInteger
             asm.code "mov ax, " + strToken
             asm.code "cwd"
             asm.code "add eax, " + strVariable2
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"

        CASE cstImmediateByte, cstImmediateInteger, cstVariableLong
             asm.code "mov eax, " + strVariable2
             asm.code "add eax, " + strToken
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"
       
    END SELECT

END SUB

SUB signed.lng.exp (strVariable AS STRING, strVariable2 AS STRING)

    expr.error "^ not implimented in this version."
    END

END SUB

SUB signed.lng.fdiv (strVariable AS STRING, strVariable2 AS STRING)
   
    expr.error "FPU division not implimented in this version."
    END

END SUB

'#############################################################################
'# Filter out all possible compile time errors.                 Long routine #
'#############################################################################
SUB signed.lng.filter
  
    SELECT CASE intToken

        CASE cstImmediateDouble, cstImmediateSingle
             expr.error "Error, type-mismatch"
             END

        CASE cstImmediateString
             expr.error "Error, type-mismatch"
             END
    
        CASE cstVariableDouble, cstVariableSingle
             expr.error "Error, type-mismatch"
             END

        CASE cstVariableString
             expr.error "Error, type-mismatch"
             END

    END SELECT

END SUB

SUB signed.lng.idiv (strVariable AS STRING, strVariable2 AS STRING)
   
    SELECT CASE intToken
          
        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "cwd"
             asm.code "mov ebx, eax"
             asm.code "mov eax, " + strVariable2
             asm.code "idiv ebx"
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"
           
        CASE cstVariableInteger
             asm.code "mov ax, " + strToken
             asm.code "cwd"
             asm.code "mov ebx, eax"
             asm.code "mov eax, " + strVariable2
             asm.code "idiv ebx"
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"

        CASE cstImmediateByte, cstImmediateInteger, cstVariableLong
             asm.code "mov eax, " + strVariable2
             asm.code "idiv " + strToken
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"

    END SELECT

END SUB

SUB signed.lng.mod (strVariable AS STRING, strVariable2 AS STRING)
   
    SELECT CASE intToken
         
        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "cwd"
             asm.code "mov ebx, eax"
             asm.code "mov eax, " + strVariable2
             asm.code "idiv ebx"
             asm.code "mov " + strVariable + ", edx"
             asm.code "jo lbl_Signed_Overflow"
          
        CASE cstVariableInteger
             asm.code "mov ax, " + strToken
             asm.code "cwd"
             asm.code "mov ebx, eax"
             asm.code "mov eax, " + strVariable2
             asm.code "idiv ebx"
             asm.code "mov " + strVariable + ", edx"
             asm.code "jo lbl_Signed_Overflow"

        CASE cstImmediateByte, cstImmediateInteger, cstVariableLong
             asm.code "mov eax, " + strVariable2
             asm.code "idiv " + strToken
             asm.code "mov " + strVariable + ", edx"
             asm.code "jo lbl_Signed_Overflow"

    END SELECT

END SUB

SUB signed.lng.mul (strVariable AS STRING, strVariable2 AS STRING)

    SELECT CASE intToken
           
        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "cwd"
             asm.code "mov ebx, eax"
             asm.code "mov eax, " + strVariable2
             asm.code "imul ebx"
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"
               
        CASE cstVariableInteger
             asm.code "mov ax, " + strToken
             asm.code "cwd"
             asm.code "mov ebx, eax"
             asm.code "mov eax, " + strVariable2
             asm.code "imul ebx"
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"

        CASE cstImmediateByte, cstImmediateInteger, cstVariableLong
             asm.code "mov eax, " + strVariable2
             asm.code "imul " + strToken
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"
      
    END SELECT

END SUB

SUB signed.lng.sub (strVariable AS STRING, strVariable2 AS STRING)
   
    SELECT CASE intToken
           
        CASE cstVariableByte
             asm.code "mov al, " + strToken
             asm.code "cbw"
             asm.code "cwd"
             asm.code "mov ebx, eax"
             asm.code "mov eax, " + strVariable2
             asm.code "sub eax, ebx"
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"
           
        CASE cstVariableInteger
             asm.code "mov ax, " + strToken
             asm.code "cwd"
             asm.code "mov ebx, eax"
             asm.code "mov eax, " + strVariable2
             asm.code "sub eax, ebx"
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"

        CASE cstImmediateByte, cstImmediateInteger, cstVariableLong
             asm.code "mov eax, " + strVariable2
             asm.code "sub eax, " + strToken
             asm.code "mov " + strVariable + ", eax"
             asm.code "jo lbl_Signed_Overflow"

    END SELECT

END SUB

